// UDPSocketAsync.cpp: implementation of the CUDPSocketAsync class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UDPSocketAsync.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CUDPSocketAsync::CUDPSocketAsync() : CUDPSocket(), CAsyncSocket()
{

}

CUDPSocketAsync::~CUDPSocketAsync()
{
	//We need to close it here
	try
	{
		Close();
	}
	ERROR_HANDLER_AMBIG(CSpoofSocket,"~CUDPSocketAsync")
}

BOOL CUDPSocketAsync::Listen()
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		int iResult;
		
		iResult=WSAAsyncSelect(GetHandle(),GetWindowHandle(),WM_SOCKET_GENERAL,FD_READ);
		if (iResult)
		{
			CUDPSocket::SetLastError("Listen");
			return FALSE;
		}

		return TRUE;
	}
	ERROR_HANDLER_AMBIG_RETURN(CSpoofSocket,"Listen",FALSE)
}

BOOL CUDPSocketAsync::SetAsync()
{
	try
	{
		//Quit if not ok
		if (!CheckSocketValid())
			return FALSE;

		//Set event to read / write 
		int iResult;

		iResult=WSAAsyncSelect(GetHandle(),GetWindowHandle(),WM_SOCKET_GENERAL,FD_WRITE | FD_READ);
		if (iResult)
		{
			CUDPSocket::SetLastError("SetAsync");
			return FALSE;
		}
		
		return TRUE;
	}
	ERROR_HANDLER_AMBIG_RETURN(CSpoofSocket,"SetAsync",FALSE)
}

BOOL CUDPSocketAsync::OnSocketConnect(int iErrorCode)
{
	//First set async again
	return SetAsync();
}

void CUDPSocketAsync::Accepted()
{
	try
	{
		AddSocketToList();

		//Go to async mode
		SetAsync();
	}
	ERROR_HANDLER_AMBIG(CSpoofSocket,"CTCPSocketAsync")
}

SOCKET CUDPSocketAsync::GetAsyncHandle()
{
	return GetHandle();
}

BOOL CUDPSocketAsync::CreateRegular()
{
	try
	{
		if (!CUDPSocket::CreateRegular())
			return FALSE;
		else
		{
			AddSocketToList();
			return TRUE;
		}
	}
	ERROR_HANDLER_AMBIG_RETURN(CSpoofSocket,"CreateRegular",FALSE)
}

BOOL CUDPSocketAsync::Create()
{
	try
	{
		if (!CUDPSocket::Create())
			return FALSE;
		else
		{
			AddSocketToList();
			return TRUE;
		}
	}
	ERROR_HANDLER_AMBIG_RETURN(CSpoofSocket,"Create",FALSE)
}

BOOL CUDPSocketAsync::Close()
{
	try
	{
		//Quit if not ok
		if (!ValidSocket())
			return FALSE;

		//Remove from socket list
		RemoveSocketFromList();

		return CUDPSocket::Close();
	}
	ERROR_HANDLER_AMBIG_RETURN(CSpoofSocket,"Close",FALSE)
}